#include "ConstString.hpp"
#include <Arbutils/Ensure.hpp>
#include <Arbutils/StringView.hpp>

static ArbUt::StringView* ConstructConstString() { return new ArbUt::StringView(); }
static ArbUt::StringView* ConstructConstStringFromStd(const std::string& s) {
    return new ArbUt::StringView(s.c_str(), s.length());
}
static ArbUt::StringView* CopyConstructConstString(ArbUt::StringView* other) {
    return new ArbUt::StringView(*other);
}
static void DestructConstString(ArbUt::StringView* self) { delete self; }
static bool ConstStringEquality(const ArbUt::StringView& a, const ArbUt::StringView& b) { return a == b; }
static bool ConstStringStdStringEquality(const ArbUt::StringView& a, const std::string& b) { return a == b; }
static uint32_t ImplConstStringHashConv(const ArbUt::StringView& s) { return s.GetHash(); }
static std::string ImplConstStringStringConv(const ArbUt::StringView& s) { return std::string(s.std_str()); }

void ConstStringRegister::Register(asIScriptEngine* engine) {
    auto r = engine->RegisterObjectType("constString", sizeof(ArbUt::StringView), asOBJ_SCOPED | asOBJ_REF);
    Ensure(r >= 0);

    r = engine->RegisterObjectBehaviour("constString", asBEHAVE_FACTORY, "constString@ f()",
                                        asFUNCTION(ConstructConstString), asCALL_CDECL);
    Ensure(r >= 0);
    r = engine->RegisterObjectBehaviour("constString", asBEHAVE_FACTORY, "constString@ f(const string &in s)",
                                        asFUNCTION(ConstructConstStringFromStd), asCALL_CDECL);
    Ensure(r >= 0);

    r = engine->RegisterObjectBehaviour("constString", asBEHAVE_FACTORY, "constString@ f(const constString &in)",
                                        asFUNCTION(CopyConstructConstString), asCALL_CDECL);
    Ensure(r >= 0);

    r = engine->RegisterObjectBehaviour("constString", asBEHAVE_RELEASE, "void f()", asFUNCTION(DestructConstString),
                                        asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);

    r = engine->RegisterObjectMethod(
        "constString", "constString &opAssign(const constString &in)",
        asMETHODPR(ArbUt::StringView, operator=, (const ArbUt::StringView&), ArbUt::StringView&), asCALL_THISCALL);
    Ensure(r >= 0);

    r = engine->RegisterObjectMethod(
        "constString", "bool opEquals(const constString &in) const",
        asFUNCTIONPR(ConstStringEquality, (const ArbUt::StringView&, const ArbUt::StringView&), bool),
        asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod(
        "constString", "bool opEquals(const string &in) const",
        asFUNCTIONPR(ConstStringStdStringEquality, (const ArbUt::StringView&, const std::string&), bool),
        asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);

    r = engine->RegisterObjectMethod("constString", "uint opImplConv()",
                                     asFUNCTIONPR(ImplConstStringHashConv, (const ArbUt::StringView&), uint32_t),
                                     asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("constString", "string opImplConv()",
                                     asFUNCTIONPR(ImplConstStringStringConv, (const ArbUt::StringView&), std::string),
                                     asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
}
